<?php

require_once 'fromemailpermission.civix.php';
// phpcs:disable
use CRM_Fromemailpermission_ExtensionUtil as E;
// phpcs:enable

define('FROMEMAIL_MAX_ITEMS', 5);

/**
 * Implements hook_civicrm_config().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_config/
 */
function fromemailpermission_civicrm_config(&$config) {
  if (isset(Civi::$statics[__FUNCTION__])) {
    return;
  }
  Civi::$statics[__FUNCTION__] = 1;
  \Civi::dispatcher()->addListener(\Civi\API\Events::RESPOND, '_fromemailpermission_civicrm_onApiRespond', -100);

  _fromemailpermission_civix_civicrm_config($config);
}

/**
 * Implements hook_civicrm_buildForm().
 */
function fromemailpermission_civicrm_buildForm($formName, &$form) {
  if ($formName == 'CRM_Admin_Form_Options') {
    if ($form->getVar('_gName') == 'from_email_address') {
      $values = $form->getVar('_values');
      if ($values['is_default']) {
        $form->assign('isDefault', TRUE);
      }
      else {

        foreach (CRM_Core_PseudoConstant::relationshipType('label') as $relationshipType) {
          $relationship_types_a_b[$relationshipType['id']] = $relationshipType['label_a_b'];
        }
        $relationship_types_a_b = [0 => E::ts(" - N/A - ")] + $relationship_types_a_b;

        $_showHide = new CRM_Core_ShowHideBlocks('', '');
        for ($i = 1; $i <= FROMEMAIL_MAX_ITEMS; $i++) {
          //the show hide blocks
          $showBlocks = 'optionField_' . $i;
          if ($i > 1) {
            $_showHide->addHide($showBlocks);
          }
          else {
            $_showHide->addShow($showBlocks);
          }
          $form->add('select', 'relationship_type_id[' . $i . ']', ts('Relationship Type'), $relationship_types_a_b);
          $form->addEntityRef('contact_b_id[' . $i . ']', ts('Contact(s)'), ['create' => TRUE, 'multiple' => TRUE]);
          $form->add('select', 'operator[' . $i . ']', ts('Operator'), ['AND' => 'AND', 'OR' => 'OR']);
        }

        if ($user_roles = CRM_Core_Config::singleton()->userSystem->getRoleNames()) {
          $roles = $form->addElement('advmultiselect',
            'roles',
            ts('Roles'),
            $user_roles,
            [
              'size' => 5,
              'style' => 'width:240px',
              'class' => 'advmultiselect',
            ]
          );
          $roles->setButtonAttributes('add', ['value' => ts('Add >>')]);
          $roles->setButtonAttributes('remove', ['value' => ts('<< Remove')]);
        }

        // set default values
        if (in_array($form->getAction(), [CRM_Core_Action::UPDATE, CRM_Core_Action::DELETE])) {
          $dao = CRM_Core_DAO::executeQuery(
            "SELECT * FROM `civicrm_fromemailpermission` WHERE id = %1;",
            [1 => [$values['value'], 'Integer']]
          );
          if ($dao->fetch()) {
            $config = unserialize($dao->config);
            $defaults['relationship_type_id'] = $config['relationship_type_id'];
            $defaults['contact_b_id'] = $config['contact_b_id'];
            $defaults['operator'] = $config['operator'];
            $defaults['roles'] = $config['roles'];

            for ($i = 1; $i <= FROMEMAIL_MAX_ITEMS; $i++) {
              if ($config['relationship_type_id'][$i]) {
                $_showHide->addShow('optionField_' . $i);
              }
            }
          }
          $form->setDefaults($defaults);
        }

        $form->assign('isDefault', FALSE);
        $_showHide->addToTemplate();
      }

      CRM_Core_Region::instance('form-bottom')->add([
        'template' => "CRM/Fromemailpermission/Admin/Form/Option.tpl",
      ]);
    }
  }
  elseif ($formName == 'CRM_Contact_Form_Task_Email') {
    // Load from_email_address again in case are cached in CRM_Core_OptionGroup::values() call
    if ($form->elementExists('from_email_address')) {
      if ($userID = CRM_Core_Session::getLoggedInContactID()) {
        $form->removeElement('from_email_address');
        $fromEmails = CRM_Fromemailpermission_BAO_Fromemailpermission::getFromEmail();
        $form->add('select', 'from_email_address', ts('From'), $fromEmails, TRUE, ['class' => 'crm-select2 huge']);
      }
    }
  }
}

/**
 * Implements hook_civicrm_optionValues().
 */
function fromemailpermission_civicrm_optionValues(&$options, $groupName) {
  if ($groupName == 'from_email_address') {
    // Filter options only for logged in Users
    if ($userID = CRM_Core_Session::getLoggedInContactID()) {
      CRM_Fromemailpermission_BAO_Fromemailpermission::getFromEmailAddresses($options, $userID);
    }
  }
}

/**
 * Implements hook_civicrm_validateForm().
 */
function fromemailpermission_civicrm_validateForm($formName, &$fields, &$files, &$form, &$errors) {
  if ($formName == 'CRM_Admin_Form_Options') {
    if ($form->getVar('_gName') == 'from_email_address') {
      foreach ($fields['relationship_type_id'] as $key => $value) {
        if ($value && !$fields['contact_b_id'][$key]) {
          $errors['contact_b_id[' . $key . ']'] = E::ts('Contact cannot be null if you selected a Relationship');
        }
        elseif (!$value && $fields['contact_b_id'][$key]) {
          $errors['relationship_type_id[' . $key . ']'] = E::ts('Relationship cannot be null if you selected a Contact');
        }
      }

      if ($fields['is_default']) {
        foreach ($fields['relationship_type_id'] as $key => $value) {
          if (!empty($fields['relationship_type_id'][$key]) || !empty($fields['contact_b_id'][$key])) {
            $errors['is_default'] = E::ts('Default FROM Email address cannot be restricted');
            return;
          }
        }
        if (!empty($fields['roles'])) {
          $errors['is_default'] = E::ts('Default FROM Email address cannot be restricted');
          return;
        }
      }

    }
  }
  return;
}

/**
 * Implements hook_civicrm_postProcess().
 */
function fromemailpermission_civicrm_postProcess($formName, $form) {
  if ($formName == 'CRM_Admin_Form_Options') {
    if ($form->getVar('_gName') == 'from_email_address') {
      $params = $form->exportValues();
      $relFields = $params['relationship_type_id'];
      foreach ($relFields as $i => $rel_id) {
        if (!$rel_id && !$params['contact_b_id'][$i]) {
          unset($params['relationship_type_id'][$i]);
          unset($params['contact_b_id'][$i]);
          unset($params['operator'][$i]);
        }
      }

      $config = [
        'relationship_type_id' => $params['relationship_type_id'],
        'contact_b_id' => $params['contact_b_id'],
        'operator' => $params['operator'],
        'roles' => $params['roles'],
      ];
      $config = serialize($config);

      if ($form->getAction() == CRM_Core_Action::DELETE) {
        if ($form->_defaultValues['value']) {
          $sql = "DELETE FROM `civicrm_fromemailpermission` WHERE id = {$form->_defaultValues['value']}";
          CRM_Core_DAO::executeQuery($sql);
        }
      }
      else {
        $sql = "
        REPLACE INTO
          `civicrm_fromemailpermission` (`id`, `config`)
        VALUES
          ({$params['value']}, '{$config}')";

        CRM_Core_DAO::executeQuery($sql);
      }

      // clear cache on changes
      \Civi\Api4\System::flush()->execute();
    }
  }
}

/**
 * Implements API Respond event
 */
function _fromemailpermission_civicrm_onApiRespond($event) {
  $request = $event->getApiRequestSig();
  $apiRequest = $event->getApiRequest();

  if ($request == '3.optionvalue.get') {
    if ($userID = CRM_Core_Session::getLoggedInContactID()) {
      $option_group_id = civicrm_api3('OptionGroup', 'getvalue', [
        'return' => "id",
        'name' => "from_email_address",
      ]);

      if (isset($apiRequest["params"]["option_group_id"]) && $apiRequest["params"]["option_group_id"] == $option_group_id) {
        // Filter options only for logged in Users
        $result = $event->getResponse();
        CRM_Fromemailpermission_BAO_Fromemailpermission::getFromEmailAddresses($result['values'], $userID, 'api');
        $result['count'] = count($result['values']);
        $event->setResponse($result);
      }
    }
  }
}

/**
 * Implements hook_civicrm_install().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_install
 */
function fromemailpermission_civicrm_install() {
  _fromemailpermission_civix_civicrm_install();
}

/**
 * Implements hook_civicrm_enable().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_enable
 */
function fromemailpermission_civicrm_enable() {
  _fromemailpermission_civix_civicrm_enable();
}
